#!@php_bin@
<?php

/*
 *       Copyright (c) 2010, Maarch SAS
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/**
 *
 * @version 0.1.0 beta
 * @author Bruno Carlin <bruno.carlin@maarch.org>
 */


#FIXME: interaction avec svn en cas de conflit...
#TODO: commands: patch
#TODO: commands: manage l10n
#TODO: accept a revision number
#TODO: Faire une classe de log


require("Maarch_CLITools/ArgsParser.php");


/********************
 ***    PARAMS    ***
 ********************/


$SVN_URL = "http://svn.maarch.org";
$MODULES = Array("advanced_physical_archive", "alert_diffusion",
                 "attachments", "autofoldering", "basket", "cases",
                 "doc_converter", "doc_project", "entities", "folder",
                 "full_text", "indexing_searching", "ldap",
                 "notes", "notifications", "ocr_tesseract",
                 "physical_archive", "postindexing", "reports",
                 "templates", "web_service", "workflow");
$MODULES_EXP = Array("moreq", "life_cycle");
$CURRENT = "branches/1.1";
$DEV = "trunk";
$SUPP_ARCHIVES = Array(".tar.gz");


/*********************
 ***    HELPERS    ***
 *********************/


class ProcessControl{
    function __construct(){
        global $options;
        $this->descriptors = array(
           // stdin is a pipe that the child will read from
           0 => array("file", "php://stdin" ,"r"),
           // stdout is a pipe that the child will write to
           1 => array("pipe", "w"),
           // stderr is a file to write to
           2 => array("file", "tmp-err.log", "a")
        );
        $this->stderr = "";
    }

    function run($cmd){
		print "running $cmd\n";
        $process = proc_open($cmd, $this->descriptors, $this->pipes);
        #while($el=stream_get_line($this->pipes[1], 8192)){
        while($el=fgets($this->pipes[1], 8192)){
            msg($el);
        }
        proc_close($process);
		$tmp = file_get_contents('tmp-err.log');
		unlink('tmp-err.log');
        if ($tmp) {
            $this->stderr .= $tmp;
        }
    }

    function get_stderr(){
        return $this->stderr;
    }
}

function print_err($msg){
    if ($msg){
        $stderr = fopen("php://stderr", "w");
        fwrite($stderr, "\n**********************************************\n"
                        ."FOLLOWING ERRORS OCCURED!\n"
                        ."**********************************************\n"
                        ."\n"
                        . $msg
                        ."**********************************************\n");
    }
}

function msg($msg){
    global $options;
    if (!$options["quiet"]){
        print $msg;
    }
}

function remove_dir($path){
    if(is_file($path)){
        unlink($path);
    }else {
        foreach(glob($path."/*") as $subpath){
            remove_dir($subpath);
        }
        foreach(glob($path."/.*") as $subpath){
            $last_part = explode("/", $subpath);
            $last_part = $last_part[count($last_part)-1];
            if (!in_array($last_part, array('.', '..'))){
                remove_dir($subpath);
            }
        }
        rmdir($path);
    }
}

function archive_tar_gz($source, $dest){

}


/******************
 ***    MAIN    ***
 ******************/


function main(){
    global $options, $DEV;
    #php build.php [-q] checkout|co [--branch=trunk] [PATH]
    #php build.php [-q] update|up [PATH]

    $checkout_cmd_parser = new ArgsParser();
    $checkout_cmd_parser->add_arg("branch", "argument", "b", "branch",
                                  $DEV, false,
         "Branch to checkout.  \"dev\" and \"trunk\" are aliases for the "
         ."development version, \"current\", \"stable\" and \"latest\" are "
         ."aliases for the latest stable version");
    $checkout_cmd_parser->add_arg("apps", "argument", "a", "apps",
                                  "maarch_entreprise", false,
                                  "Application to checkout");
    $checkout_cmd_parser->add_arg("experimental", "switch", "e",
                                  "with-experimental", false, false,
                                  "Checkout experimental modules too");
    $checkout_cmd_parser->add_arg("path", "positional", null, null,
                                  getcwd(), false,
         "Path the working copy will be checked out to");

    $update_cmd_parser = new ArgsParser();
    $update_cmd_parser->add_arg("path", "positional", null, null,
                                getcwd(), false,
                                "Path of the working copy to update");

    $sql_cmd_parser = new ArgsParser();
    $sql_cmd_parser->add_arg("path", "positional", null, null,
                                getcwd(), false,
                                "Path of the working copy for wich SQL "
                                ."scripts must be generated");
    $sql_cmd_parser->add_arg("dbengine", "argument", "d", "dbengine",
                             "postgresql", false,
                              "Database engine ");

    $dist_cmd_parser = new ArgsParser();
    $dist_cmd_parser->add_arg("tmppath", "argument", "t", "tmp-dir",
                              getcwd(), false,
                              "Create the tmp dir here");
    $dist_cmd_parser->add_arg("filename", "positional", null, null,
                              null, true,
                              "Path and name of the package");
    $dist_cmd_parser->add_arg("path", "positional", null, null,
                              getcwd(), false,
                              "Path of the working copy to package");

    $branch_cmd_parser = new ArgsParser();
    $branch_cmd_parser->add_arg("branch", "argument", "b", "branch",
                                $DEV, false,
         "The path to branch. \"dev\" and \"trunk\" are aliases for the "
         ."development version, \"current\", \"stable\" and \"latest\" are "
         ."aliases for the latest stable version");
    $branch_cmd_parser->add_arg("branch_name", "positional", null, null,
                                null, true,
                                "Name of the new branch");

    $tag_cmd_parser = new ArgsParser();
    $tag_cmd_parser->add_arg("branch", "argument", "b", "branch",
                             $DEV, false,
         "The path to tag. \"dev\" and \"trunk\" are aliases for the "
         ."development version, \"current\", \"stable\" and \"latest\" are "
         ."aliases for the latest stable version");
    $tag_cmd_parser->add_arg("tag_name", "positional", null, null,
                             null, true,
                             "Name of the new branch");

    $arg_parser = new ArgsParser();
    $arg_parser->add_arg("quiet", "switch", "q", "quiet",
                         false, false,
                         "Disable all output by the program");
    $arg_parser->add_command("checkout", Array("co"), $checkout_cmd_parser,
                         "Create a new working copy of the application");
    $arg_parser->add_command("update", Array("up"), $update_cmd_parser,
                         "Perform a recursive update on the working copy");
    $arg_parser->add_command("dist", Array(), $dist_cmd_parser,
                         "Create a package of the application");
    $arg_parser->add_command("tag", Array(), $tag_cmd_parser,
                         "Create a new tag for the application");
    $arg_parser->add_command("branch", Array(), $branch_cmd_parser,
                         "Create a new branch for the application");
    $arg_parser->add_command("sql", Array(), $sql_cmd_parser,
                         "Generate SQL scripts for the application");
    $options = $arg_parser->parse_args($GLOBALS["argv"]);

    if (!array_key_exists("command", $options)){
        msg("Aucune commande n'a été fournie !");
        $arg_parser->usage();
    }



    dispatch($options["command"]);
}

function dispatch($command){
    if (function_exists("do_".$command["name"])){
        call_user_func("do_".$command["name"], $command["opts"]);
    }
}


/*********************
 ***    ACTIONS    ***
 *********************/

function do_checkout($opts){
    global $SVN_URL, $MODULES, $CURRENT, $DEV;

    switch ($opts['branch']){
        case "dev":
        case "trunk":
            $branch = $DEV;
            break;

        case "current":
        case "latest":
        case "stable":
            $branch = $CURRENT;
            break;

        default:
            $branch = $opts['branch'];
    }

    if ($opts["experimental"] && $branch == $DEV){
        global $MODULES_EXP;
        array_splice($MODULES, count($MODULES), 0, $MODULES_EXP);
    }

    $pc = new ProcessControl();
    msg("\n* checking out core\n");
    $pc->run(sprintf("svn co \"%s/core/%s\" \"%s\"", $SVN_URL, $branch,
                                             $opts["path"]));
    msg("\n* checking out apps\n");
    $pc->run(sprintf("svn co \"%s/%s/%s\" \"%s/apps/%s\"", $SVN_URL, $opts["apps"],
                                                $branch, $opts["path"], $opts["apps"]));

    foreach($MODULES as $module){
        msg("\n* checking out module $module\n");
        $pc->run(sprintf("svn co \"%s/%s/%s\" \"%s/modules/%s\"", $SVN_URL, $module,
                                                       $branch, $opts["path"], $module));
    }
    print_err($pc->stderr);
}

function do_update($opts){

    chdir($opts["path"]);

    $pc = new ProcessControl();

    msg("* updating core\n");
    $pc->run("svn up");

    msg("* updating apps");
    $pc->run("\nsvn up \"apps/*\"\n");

    foreach(scandir("modules") as $module){
        msg("\n* updating \"modules/$module\"\n");
        $pc->run("svn up \"modules/$module\"");
    }
    print_err($pc->stderr);
}

function do_dist($opts){
    global $SUPP_ARCHIVES;

    $filepath = $opts['filename'];
    $filename = explode(DIRECTORY_SEPARATOR, $filepath);
    $filename = $filename[count($filename)-1];
    foreach($SUPP_ARCHIVES as $filetype){
        if(strrpos($filetype, $filename)){
            $filetype = $filetype;
            break;
        }
    }
    $basename = str_replace($filetype, "", $filename);
    $tmpdir = $opts["tmppath"]."/".$basename;
    $wcpath = realpath($opts["path"]);

    mkdir($tmpdir, 0777, True);

    chdir($wcpath);

    $pc = new ProcessControl();

    $pc->run(sprintf("svn export \"%s\" \"%s/\"", '.', $tmpdir));

    mkdir($tmpdir.'/apps');
    $apps = scandir("apps");
    $apps = $apps[2];
    $pc->run(sprintf("svn export \"apps/%s\" \"%s/%s/apps/%s\"",
                     $apps, $tmpdir, $basename, $apps));

    mkdir($tmpdir.'/modules');
    foreach(glob("modules/*") as $module){
        $pc->run(sprintf("svn export \"%s\" \"%s/%s/%s\"",
                         $module, $tmpdir, $basename, $module));
    }

    chdir($tmpdir);
    switch($filetype){
        case ".tar.gz":
            $pc->run("tar czf $filepath $basename");
            break;

        case ".tar.bz2":
            $pc->run("tar cjf $filepath $basename");
            break;

        case ".tar.lzma":
            $pc->run("tar -cf --lzma $filepath $basename");
            break;

        case ".tar.xz":
            $pc->run("tar -cf --xz $filepath $basename");
            break;

        case ".zip":
            $pc->run("zip -r $filepath $basename");
            break;

        default:
            rename($basename, $filepath);
            break;
    }


    chdir($wcpath);

    remove_dir($tmpdir);

    print_err($pc->stderr);
}


function do_branch($opts){
    global $SVN_URL, $DEV, $CURRENT, $MODULES, $MODULES_EXP;

    switch ($opts['branch']){
        case "dev":
        case "trunk":
            $branch = $DEV;
            break;

        case "current":
        case "latest":
        case "stable":
            $branch = $CURRENT;
            break;

        default:
            $branch = $opts['branch'];
    }

    array_splice($MODULES, count($MODULES), 0, $MODULES_EXP);

    $pc = new ProcessControl();
    $pc->run(sprintf("svn branch \"%s/core/%s\" \"%s/core/branches/%s\"",
                      $SVN_URL, $branch, $SVN_URL, $opts["branch_name"]));
    $pc->run(sprintf("svn branch \"%s/maarch_entreprise/%s\" "
                     ."\"%s/maarch_entreprise/branches/%s\"",
                      $SVN_URL, $branch, $SVN_URL, $opts["branch_name"]));
    foreach($MODULES as $module){
        $pc->run(sprintf("svn branch \"%s/%s/%s\" \"%s/%s/branches/%s\"",
                         $SVN_URL, $module, $branch, $SVN_URL, $module,
                         $opts["branch_name"]));
    }
}


function do_tag($opts){
    global $SVN_URL, $DEV, $CURRENT, $MODULES, $MODULES_EXP;

    switch ($opts['branch']){
        case "dev":
        case "trunk":
            $branch = $DEV;
            break;

        case "current":
        case "latest":
        case "stable":
            $branch = $CURRENT;
            break;

        default:
            $branch = $opts['branch'];
    }

    array_splice($MODULES, count($MODULES), 0, $MODULES_EXP);

    $pc = new ProcessControl();
    $pc->run(sprintf("svn tag \"%s/core/%s\" \"%s/core/tags/%s\"",
                      $SVN_URL, $branch, $SVN_URL, $opts["tag_name"]));
    $pc->run(sprintf("svn tag \"%s/maarch_entreprise/%s\" "
                     ."\"%s/maarch_entreprise/tags/%s\"",
                     $SVN_URL, $branch, $SVN_URL, $opts["tag_name"]));
    foreach($MODULES as $module){
        $pc->run(sprintf("svn tag \"%s/%s/%s\" \"%s/%s/tags/%s\"",
                      $SVN_URL, $module, $branch, $SVN_URL, $module,
                      $opts["tag_name"]));
    }
}

function do_sql($opts){

    chdir($opts["path"]);

    $pc = new ProcessControl();

    $datafile = fopen("data.sql", "w");
    $structurefile = fopen("structure.sql", "w");

    if (file_exists("core/sql/data/core.${opts['dbengine']}.sql")){
        fwrite($datafile, "\n\n-- core/sql/data/core.${opts['dbengine']}.sql\n\n");
        fwrite($datafile, file_get_contents("core/sql/data/core.${opts['dbengine']}.sql"));
    }
    if (file_exists("core/sql/structure/core.${opts['dbengine']}.sql")){
        fwrite($structurefile, "\n\n-- core/sql/structure/core.${opts['dbengine']}.sql\n\n");
        fwrite($structurefile, file_get_contents("core/sql/structure/core.${opts['dbengine']}.sql"));
    }


    $apps = scandir("apps");
    $apps = $apps[2];
    if (file_exists("apps/$apps/sql/data/apps.${opts['dbengine']}.sql")){
        fwrite($datafile, "\n\n-- apps/$apps/sql/data/apps.${opts['dbengine']}.sql\n\n");
        fwrite($datafile, file_get_contents("apps/$apps/sql/data/apps.${opts['dbengine']}.sql"));
    }
    if (file_exists("apps/$apps/sql/structure/apps.${opts['dbengine']}sql")){
        fwrite($structurefile, "\n\n-- apps/$apps/sql/structure/apps.${opts['dbengine']}.sql\n\n");
        fwrite($structurefile, file_get_contents("apps/$apps/sql/structure/apps.${opts['dbengine']}.sql"));
    }


    foreach(glob("modules/*") as $module){
        $module_name = explode("/", $module);
        $module_name = $module_name[1];
        if (file_exists("$module/sql/data/$module_name.${opts['dbengine']}.sql")){
            fwrite($datafile, "\n\n-- $module/sql/data/$module_name.${opts['dbengine']}.sql\n\n");
            fwrite($datafile, file_get_contents("$module/sql/data/$module_name.${opts['dbengine']}.sql"));
        }
        if (file_exists("$module/sql/structure/$module_name.${opts['dbengine']}.sql")){
            fwrite($structurefile, "\n\n-- $module/sql/structure/$module_name.${opts['dbengine']}.sql\n\n");
            fwrite($structurefile, file_get_contents("$module/sql/structure/$module_name.${opts['dbengine']}.sql"));
        }
    }

    $err_msg = $pc->stderr;
    if (strlen($err_msg)){
        print_err($err_msg);
    }else{
        msg("the files data.sql and structure.sql have been generated successfully!\n");
    }

}

main();